@@ -2,12 +2,14 @@ from pythoscope.astvisitor import descend, ASTVisitor
 from pythoscope.astbuilder import parse_fragment, EmptyCode
 from pythoscope.logger import log
 from pythoscope.generator.adder import add_test_case_to_project
+from pythoscope.generator.code_string import CodeString, combine, join, putinto
+from pythoscope.generator.selector import testable_objects, is_testable, \
+    testable_calls
 from pythoscope.serializer import BuiltinException, CompositeObject, \
     ImmutableObject, MapObject, UnknownObject, SequenceObject, \
     SerializedObject, can_be_constructed, is_serialized_string
 from pythoscope.store import Class, Function, FunctionCall, TestClass, \
-    TestMethod, ModuleNotFound, UserObject, MethodCall, Method, Project, \
-    GeneratorObject
+    TestMethod, ModuleNotFound, UserObject, MethodCall, Method, GeneratorObject
 from pythoscope.compat import any, set, sorted
 from pythoscope.util import camelize, compact, counted, flatten, \
     key_for_value, pluralize, underscore, union
@@ -35,28 +37,7 @@ def todo_value(value):
     """
     return "<TODO: %s>" % value
 
-class CallString(str):
-    """A string that holds information on the function/method call it
-    represents.
-
-    `uncomplete` attribute denotes whether it is a complete call
-    or just a template.
-
-    `imports` is a list of imports that this call requires.
-    """
-    def __new__(cls, string, uncomplete=False, imports=None):
-        if imports is None:
-            imports = set()
-        call_string = str.__new__(cls, string)
-        call_string.uncomplete = uncomplete
-        call_string.imports = imports
-        return call_string
-
-    def extend(self, value, uncomplete=False, imports=set()):
-        return CallString(value, self.uncomplete or uncomplete,
-                          self.imports.union(imports))
-
-# :: (SerializedObject | [SerializedObject], {SerializedObject: str}) -> CallString
+# :: (SerializedObject | [SerializedObject], {SerializedObject: str}) -> CodeString
 def constructor_as_string(object, assigned_names={}):
     """For a given object (either a SerializedObject or a list of them) return
     a string representing a code that will construct it.
@@ -78,15 +59,15 @@ def constructor_as_string(object, assigned_names={}):
         'SomeClass()'
 
     interpreting their arguments correctly
-        >>> obj.add_call(MethodCall(Method('__init__'), {'arg': serialize('whatever')}, serialize(None)))
+        >>> obj.add_call(MethodCall(Method('__init__', ['self', 'arg']), {'arg': serialize('whatever')}, serialize(None)))
         >>> constructor_as_string(obj)
-        "SomeClass(arg='whatever')"
+        "SomeClass('whatever')"
 
     even if they're user objects themselves:
         >>> otherobj = UserObject(None, Class('SomeOtherClass'))
-        >>> otherobj.add_call(MethodCall(Method('__init__'), {'object': obj}, serialize(None)))
+        >>> otherobj.add_call(MethodCall(Method('__init__', ['self', 'object']), {'object': obj}, serialize(None)))
         >>> constructor_as_string(otherobj)
-        "SomeOtherClass(object=SomeClass(arg='whatever'))"
+        "SomeOtherClass(SomeClass('whatever'))"
 
     Handles composite objects:
         >>> constructor_as_string(serialize([1, "a", None]))
@@ -94,7 +75,7 @@ def constructor_as_string(object, assigned_names={}):
 
     even when they contain instances of user-defined classes:
         >>> constructor_as_string(SequenceObject([obj], lambda x:x))
-        "[SomeClass(arg='whatever')]"
+        "[SomeClass('whatever')]"
 
     or other composite objects:
         >>> constructor_as_string(serialize((23, [4, [5]], {'a': 'b'})))
@@ -107,72 +88,158 @@ def constructor_as_string(object, assigned_names={}):
     if isinstance(object, list):
         return list_of(map(constructor_as_string, object))
     elif assigned_names.has_key(object):
-        return CallString(assigned_names[object])
+        return CodeString(assigned_names[object])
     elif isinstance(object, UserObject):
-        args = {}
         # Look for __init__ call and base the constructor on that.
         init_call = object.get_init_call()
         if init_call:
-            args = init_call.input
-        return call_as_string(object.klass.name, args,
-                              init_call and init_call.definition)
+            return call_as_string_for(object.klass.name, init_call.input,
+                init_call.definition)
+        else:
+            return call_as_string(object.klass.name, {})
     elif isinstance(object, ImmutableObject):
-        return CallString(object.reconstructor, imports=object.imports)
+        return CodeString(object.reconstructor, imports=object.imports)
     elif isinstance(object, CompositeObject):
-        try:
-            reconstructors, imports, uncomplete = zip(*get_contained_objects_info(object, assigned_names))
-        # In Python <= 2.3 zip can raise TypeError if no arguments were provided.
-        # All Pythons can raise ValueError because of the wrong unpacking.
-        except (ValueError, TypeError):
-            reconstructors, imports, uncomplete = [], [], []
-        return CallString(object.constructor_format % ', '.join(reconstructors),
-                          imports=union(object.imports, *imports),
-                          uncomplete=any(uncomplete))
+        arguments = join(', ', get_contained_objects_info(object, assigned_names))
+        return putinto(arguments, object.constructor_format, object.imports)
     elif isinstance(object, UnknownObject):
-        return CallString(todo_value(object.partial_reconstructor), uncomplete=True)
+        return CodeString(todo_value(object.partial_reconstructor), uncomplete=True)
     else:
         raise TypeError("constructor_as_string expected SerializedObject at input, not %s" % object)
 
-# :: ([SerializedObject], {SerializedObject: str}) -> [(str, set, bool)]
+# :: ([SerializedObject], {SerializedObject: str}) -> [CodeString]
 def get_objects_collection_info(objs, assigned_names):
     for obj in objs:
-        cs = constructor_as_string(obj, assigned_names)
-        yield (cs, cs.imports, cs.uncomplete)
+        yield constructor_as_string(obj, assigned_names)
 
-# :: ({SerializedObject: SerializedObject}, {SerializedObject: str}) -> [(str, set, bool)]
+# :: ({SerializedObject: SerializedObject}, {SerializedObject: str}) -> [CodeString]
 def get_objects_mapping_info(mapping, assigned_names):
     for key, value in mapping:
         keycs = constructor_as_string(key, assigned_names)
         valuecs = constructor_as_string(value, assigned_names)
-        yield ("%s: %s" % (keycs, valuecs),
-               union(keycs.imports, valuecs.imports),
-               keycs.uncomplete or valuecs.uncomplete)
+        yield combine(keycs, valuecs, "%s: %s")
 
-# :: (CompositeObject, {SerializedObject: str}) -> [(str, set, bool)]
+# :: (CompositeObject, {SerializedObject: str}) -> [CodeString]
 def get_contained_objects_info(obj, assigned_names):
-    """Return a list of tuples (reconstructor, imports, uncomplete) describing
-    each object contained within a composite object.
+    """Return a list of CodeStrings describing each object contained within
+    a composite object.
     """
     if isinstance(obj, SequenceObject):
-        return get_objects_collection_info(obj.contained_objects, assigned_names)
+        return list(get_objects_collection_info(obj.contained_objects, assigned_names))
     elif isinstance(obj, MapObject):
-        return get_objects_mapping_info(obj.mapping, assigned_names)
+        return list(get_objects_mapping_info(obj.mapping, assigned_names))
     elif isinstance(obj, BuiltinException):
-        return get_objects_collection_info(obj.args, assigned_names)
+        return list(get_objects_collection_info(obj.args, assigned_names))
     else:
         raise TypeError("Wrong argument to get_contained_objects_info: %r." % obj)
 
-# :: (string, dict, Definition, {SerializedObject: str}) -> CallString
-def call_as_string(object_name, args, definition=None, assigned_names={}):
+# :: Definition -> [str]
+def arguments_of(definition):
+    if isinstance(definition, Method):
+        return definition.args[1:] # Skip "self".
+    return definition.args
+
+# :: (string, dict, Definition, {SerializedObject: str}) -> CodeString
+def call_as_string_for(object_name, args, definition, assigned_names={}):
     """Generate code for calling an object with given arguments.
 
     >>> from test.helper import make_fresh_serialize
     >>> serialize = make_fresh_serialize()
 
-    >>> call_as_string('fun', {'a': serialize(1), 'b': serialize(2)})
-    'fun(a=1, b=2)'
-    >>> call_as_string('capitalize', {'str': serialize('string')})
-    "capitalize(str='string')"
+    Puts varargs at the end of arguments list.
+        >>> call_as_string_for('build_url',
+        ...     {'proto': serialize('http'), 'params': serialize(('user', 'session', 'new'))},
+        ...     Function('build_url', ['proto', '*params']))
+        "build_url('http', 'user', 'session', 'new')"
+
+    Works for lone varargs too.
+        >>> call_as_string_for('concat', {'args': serialize(([1,2,3], [4,5], [6]))},
+        ...     Function('concat', ['*args']))
+        'concat([1, 2, 3], [4, 5], [6])'
+
+    Uses assigned name for varargs as well.
+        >>> args = serialize((1, 2, 3))
+        >>> call_as_string_for('add', {'args': args}, Function('add', ['*args']), {args: 'atuple'})
+        'add(*atuple)'
+
+    Inlines extra keyword arguments in the call...
+        >>> call_as_string_for('dict', {'kwargs': serialize({'one': 1, 'two': 2})},
+        ...     Function('dict', ['**kwargs']))
+        'dict(one=1, two=2)'
+
+    ...even when they are combined with varargs.
+        >>> call_as_string_for('wrap', {'a': serialize((1, 2, 3)), 'k': serialize({'x': 4, 'y': 5})},
+        ...     Function('wrap', ['*a', '**k']))
+        'wrap(1, 2, 3, x=4, y=5)'
+
+    Uses assigned name for kwarg if present.
+        >>> kwargs = serialize({'id': 42, 'model': 'user'})
+        >>> call_as_string_for('filter_params', {'kwargs': kwargs},
+        ...    Function('filter_params', ['**kwargs']), {kwargs: 'params'})
+        'filter_params(**params)'
+
+    Generates valid code when vararg has been named and kwarg wasn't.
+        >>> args = serialize((1, 2, 3))
+        >>> call_as_string_for('wrap', {'args': args, 'kwargs': serialize({'a': 6, 'b': 7})},
+        ...     Function('wrap', ['*args', '**kwargs']), {args: 'atuple'})
+        'wrap(a=6, b=7, *atuple)'
+
+    When varargs are present all preceding arguments are positioned, not named.
+        >>> call_as_string_for('sum', {'x': serialize(1), 'rest': serialize((2, 3))},
+        ...     Function('sum', ['x', '*rest']))
+        'sum(1, 2, 3)'
+    """
+    positional_args = []
+    keyword_args = []
+    vararg = None
+    kwarg = None
+
+    def getvalue(argname):
+        return args[argname.lstrip("*")]
+
+    skipped_an_arg = False
+    for argname in arguments_of(definition):
+        try:
+            value = getvalue(argname)
+            if argname.startswith("**"):
+                if value in assigned_names.keys():
+                    kwarg = CodeString("**%s" % assigned_names[value])
+                else:
+                    for karg, kvalue in map_as_kwargs(value):
+                        valuecs = constructor_as_string(kvalue, assigned_names)
+                        keyword_args.append(combine(karg, valuecs, "%s=%s"))
+            elif argname.startswith("*"):
+                if value in assigned_names.keys():
+                    vararg = CodeString("*%s" % assigned_names[value])
+                else:
+                    code_strings = get_contained_objects_info(value, assigned_names)
+                    positional_args.extend(code_strings)
+            else:
+                constructor = constructor_as_string(value, assigned_names)
+                if skipped_an_arg:
+                    keyword_args.append(combine(argname, constructor, "%s=%s"))
+                else:
+                    positional_args.append(constructor)
+        except KeyError:
+            skipped_an_arg = True
+
+    arguments = join(', ', filter(None, (positional_args + keyword_args + [vararg] + [kwarg])))
+    return combine(object_name, arguments, "%s(%s)")
+
+# :: (string, dict, {SerializedObject: str}) -> CodeString
+def call_as_string(object_name, args, assigned_names={}):
+    """Generate code for calling an arbitrary object with given arguments.
+    Use `call_as_string_for` when you have a definition to base the call on.
+
+    >>> from test.helper import make_fresh_serialize
+    >>> serialize = make_fresh_serialize()
+
+    Since we don't have a definition to base the generated call on, we use
+    keywords to name all arguments:
+        >>> call_as_string('fun', {'a': serialize(1), 'b': serialize(2)})
+        'fun(a=1, b=2)'
+        >>> call_as_string('capitalize', {'str': serialize('string')})
+        "capitalize(str='string')"
 
     Uses references to existing objects where possible...
         >>> result = call_as_string('call', {'f': serialize(call_as_string)})
@@ -193,70 +260,18 @@ def call_as_string(object_name, args, definition=None, assigned_names={}):
     construction code.
         >>> mutable = serialize([])
         >>> call_as_string('merge', {'seq1': mutable, 'seq2': serialize([1,2,3])},
-        ...     None, {mutable: 'alist'})
+        ...     {mutable: 'alist'})
         'merge(seq1=alist, seq2=[1, 2, 3])'
-
-    Puts varargs at the end of arguments list.
-        >>> call_as_string('build_url',
-        ...     {'proto': serialize('http'), 'params': serialize(('user', 'session', 'new'))},
-        ...     Function('build_url', ['proto', '*params']))
-        "build_url(proto='http', 'user', 'session', 'new')"
-
-    Works for lone varargs too.
-        >>> call_as_string('concat', {'args': serialize(([1,2,3], [4,5], [6]))},
-        ...     Function('concat', ['*args']))
-        'concat([1, 2, 3], [4, 5], [6])'
-
-    Uses assigned name for varargs as well.
-        >>> args = serialize((1, 2, 3))
-        >>> call_as_string('add', {'args': args}, Function('add', ['*args']), {args: 'atuple'})
-        'add(*atuple)'
-
-    Inlines extra keyword arguments in the call...
-        >>> call_as_string('dict', {'kwargs': serialize({'one': 1, 'two': 2})},
-        ...     Function('dict', ['**kwargs']))
-        'dict(one=1, two=2)'
-
-    ...even when they are combined with varargs.
-        >>> call_as_string('wrap', {'a': serialize((1, 2, 3)), 'k': serialize({'x': 4, 'y': 5})},
-        ...     Function('wrap', ['*a', '**k']))
-        'wrap(1, 2, 3, x=4, y=5)'
-
-    Uses assigned name for kwarg if present.
-        >>> kwargs = serialize({'id': 42, 'model': 'user'})
-        >>> call_as_string('filter_params', {'kwargs': kwargs},
-        ...    Function('filter_params', ['**kwargs']), {kwargs: 'params'})
-        'filter_params(**params)'
     """
-    arguments = []
-    varargs = []
-    kwargs = []
     uncomplete = False
     imports = set()
+    arguments = []
     for arg, value in sorted(args.iteritems()):
-        if definition and definition.is_vararg(arg):
-            if value in assigned_names.keys():
-                varargs = ["*%s" % assigned_names[value]]
-            else:
-                rec, imp, unc = zip(*get_contained_objects_info(value, assigned_names))
-                uncomplete = uncomplete or any(unc)
-                imports.update(union(*imp))
-                varargs = list(rec)
-        elif definition and definition.is_kwarg(arg):
-            if value in assigned_names.keys():
-                kwargs = ["**%s" % assigned_names[value]]
-            else:
-                for karg, kvalue in map_as_kwargs(value):
-                    valuecs = constructor_as_string(kvalue, assigned_names)
-                    uncomplete = uncomplete or valuecs.uncomplete
-                    imports.update(valuecs.imports)
-                    kwargs.append("%s=%s" % (karg, valuecs))
-        else:
-            constructor = constructor_as_string(value, assigned_names)
-            uncomplete = uncomplete or constructor.uncomplete
-            imports.update(constructor.imports)
-            arguments.append("%s=%s" % (arg, constructor))
-    return CallString("%s(%s)" % (object_name, ', '.join(arguments + varargs + kwargs)),
+        constructor = constructor_as_string(value, assigned_names)
+        uncomplete = uncomplete or constructor.uncomplete
+        imports.update(constructor.imports)
+        arguments.append("%s=%s" % (arg, constructor))
+    return CodeString("%s(%s)" % (object_name, ', '.join(arguments)),
                       uncomplete=uncomplete, imports=imports)
 
 # :: MapObject -> {str: SerializedObject}
@@ -386,21 +401,19 @@ def assign_names_for(context):
 def assigned_names_sorted_by_timestamp(items):
     return sorted(items, key=lambda i: i[0].timestamp)
 
-# :: {SerializedObject: str} -> CallString
+# :: {SerializedObject: str} -> CodeString
 def create_setup_for_named_objects(assigned_names):
-    full_setup = CallString("")
+    full_setup = CodeString("")
     already_assigned_names = {}
     # Note that since data we have was gathered during real execution there is
     # no way setup dependencies are cyclic, i.e. there is a strict order of
     # object creation. We've chosen to sort objects by their creation timestamp.
     for obj, name in assigned_names_sorted_by_timestamp(assigned_names.iteritems()):
         constructor = constructor_as_string(obj, already_assigned_names)
-        setup = "%s = %s\n" % (name, constructor)
+        setup = combine(name, constructor, "%s = %s\n")
         if constructor.uncomplete:
-            setup = "# %s" % setup
-        full_setup = full_setup.extend("%s%s" % (full_setup, setup),
-                                       constructor.uncomplete,
-                                       constructor.imports)
+            setup = combine("# ", setup)
+        full_setup = combine(full_setup, setup)
         already_assigned_names[obj] = name
     return full_setup
 
@@ -528,9 +541,25 @@ def call2testname(call, object_name):
     else:
         return objcall2testname(object_name, call.input, call.output)
 
+# :: [TestMethodDescription] -> [TestMethodDescription]
 def sorted_test_method_descriptions(descriptions):
     return sorted(descriptions, key=lambda md: md.name)
 
+# :: [TestMethodDescription] -> [TestMethodDescription]
+def resolve_name_duplicates(descriptions):
+    # We abuse the fact that descriptions has been sorted by name before being
+    # passed into this function.
+    last_name = ''
+    num = 2
+    for description in descriptions:
+        if last_name != description.name:
+            last_name = description.name
+            num = 2
+        else:
+            description.name = "%s_case_%d" % (description.name, num)
+            num += 1
+    return descriptions
+
 def name2testname(name):
     if name[0].isupper():
         return "Test%s" % name
@@ -579,24 +608,21 @@ def class_init_stub(klass):
         args = init_method.get_call_args()
     return call_with_args(klass.name, args)
 
-# :: (Call, CallString) -> CallString
+# :: (Call, CodeString) -> CodeString
 def decorate_call(call, string):
     if isinstance(call, GeneratorObject):
         invocations = len(call.output)
         if call.raised_exception():
             invocations += 1
         # TODO: generators were added to Python 2.2, while itertools appeared in
-        # release  2.3, so we may generate incompatible tests here.
-        return string.extend("list(islice(%s, %d))" % (string, invocations),
-                             imports=[("itertools", "islice")])
+        # release 2.3, so we may generate incompatible tests here.
+        return putinto(string, "list(islice(%%s, %s))" % invocations,
+            set([("itertools", "islice")]))
     return string
 
 def should_ignore_method(method):
     return method.is_private()
 
-def testable_calls(calls):
-    return [c for c in calls if c.is_testable()]
-
 class UnknownTemplate(Exception):
     def __init__(self, template):
         Exception.__init__(self, "Couldn't find template %r." % template)
@@ -673,8 +699,12 @@ class TestGenerator(object):
 
     def add_tests_to_project(self, project, modnames, force=False):
         for modname in modnames:
-            module = project.find_module_by_full_path(modname)
-            self._add_tests_for_module(module, project, force)
+            try:
+                module = project.find_module_by_full_path(modname)
+                if not module.has_errors():
+                    self._add_tests_for_module(module, project, force)
+            except ModuleNotFound:
+                log.warning("Failed to inspect module %s, skipping test generation." % modname)
 
     def comment_assertion(self, comment):
         return comment
@@ -691,14 +721,14 @@ class TestGenerator(object):
             add_test_case_to_project(project, test_case, self.main_snippet, force)
 
     def _generate_test_cases(self, module):
-        for object in module.testable_objects:
+        for object in testable_objects(module):
             test_case = self._generate_test_case(object, module)
             if test_case:
                 yield test_case
 
     def _generate_test_case(self, object, module):
         class_name = name2testname(camelize(object.name))
-        method_descriptions = sorted_test_method_descriptions(self._generate_test_method_descriptions(object, module))
+        method_descriptions = resolve_name_duplicates(sorted_test_method_descriptions(self._generate_test_method_descriptions(object, module)))
 
         # Don't generate empty test classes.
         if method_descriptions:
@@ -752,7 +782,7 @@ class TestGenerator(object):
             # We have at least one call registered, so use it.
             return self._method_descriptions_from_function(function)
         else:
-            # No calls were traced, so we're go for a single test stub.
+            # No calls were traced, so we'll go for a single test stub.
             log.debug("Detected _no_ testable calls in function %s." % function.name)
             name = name2testname(underscore(function.name))
             assertions = [assertion_stub(function.name, function.args), ('missing',)]
@@ -866,7 +896,7 @@ class TestGenerator(object):
         Generated assertion will be a stub if input of a call cannot be
         constructed or if stub argument is True.
         """
-        callstring = decorate_call(call, call_as_string(name, call.input,
+        callstring = decorate_call(call, call_as_string_for(name, call.input,
             call.definition, assigned_names))
 
         self.ensure_imports(callstring.imports)
