@@ -1,11 +1,28 @@
 import array
+import exceptions
 import re
-import sets
 import types
 
-from pythoscope.util import RePatternType, all, class_name, frozenset, \
-    module_name, regexp_flags_as_string, set, string2id, underscore
+from pythoscope.compat import all, frozenset, set, sets
+from pythoscope.util import RePatternType, class_name, class_of, \
+    module_name, regexp_flags_as_string, string2id, underscore
 
+# Filter out private attributes, like __doc__, __name__ and __package__.
+BUILTIN_EXCEPTION_TYPES = set([v for k,v in exceptions.__dict__.items() if not k.startswith('_')])
+
+# Exceptions with special semantics for the `args` attribute.
+# See <http://docs.python.org/library/exceptions.html#exceptions.EnvironmentError>
+# for details.
+BUILTIN_ENVIRONMENT_ERROR_TYPES = [EnvironmentError, OSError, IOError]
+
+# Include VMSError or WindowsError if they exist.
+try:
+    BUILTIN_ENVIRONMENT_ERROR_TYPES.append(WindowsError)
+except NameError:
+    try:
+        BUILTIN_ENVIRONMENT_ERROR_TYPES.append(VMSError)
+    except NameError:
+        pass
 
 # :: SerializedObject | [SerializedObject] -> bool
 def can_be_constructed(obj):
@@ -24,19 +41,19 @@ def get_human_readable_id(obj):
         return 'false'
 
     # ... based on object's type,
-    objtype = type(obj)
+    objclass = class_of(obj)
     mapping = {list: 'list',
                dict: 'dict',
                tuple: 'tuple',
                unicode: 'unicode_string',
                types.GeneratorType: 'generator'}
-    objid = mapping.get(objtype)
+    objid = mapping.get(objclass)
     if objid:
         return objid
 
     # ... or based on its supertype.
     if isinstance(obj, Exception):
-        return underscore(objtype.__name__)
+        return underscore(objclass.__name__)
     elif isinstance(obj, RePatternType):
         return "%s_pattern" % string2id(obj.pattern)
     elif isinstance(obj, types.FunctionType):
@@ -51,7 +68,7 @@ def get_human_readable_id(obj):
             string = "<>"
         # Looks like an instance without a custom __str__ defined.
         if string.startswith("<"):
-            return "%s_instance" % underscore(objtype.__name__)
+            return "%s_instance" % underscore(objclass.__name__)
         else:
             return string2id(string)
 
@@ -101,6 +118,8 @@ class SerializedObject(object):
     def __init__(self, obj):
         self.timestamp = SerializedObject.next_timestamp()
         self.human_readable_id = get_human_readable_id(obj)
+        self.module_name = module_name(obj)
+        self.type_name = get_type_name(obj)
 
     def _get_type_import(self):
         if self.module_name not in ['__builtin__', 'exceptions']:
@@ -140,15 +159,23 @@ class ImmutableObject(SerializedObject):
         return hash(self.reconstructor)
 
     def __repr__(self):
-        return "ImmutableObject(%r, imports=%r)" % (self.reconstructor, self.imports)
+        if self.imports:
+            return "ImmutableObject(%r, imports=%r)" % (self.reconstructor, self.imports)
+        return "ImmutableObject(%r)" % self.reconstructor
 
     # :: object -> (string, set)
     def get_reconstructor_with_imports(obj):
         """
-        >>> ImmutableObject.get_reconstructor_with_imports(re.compile('abcd'))
-        ("re.compile('abcd')", set(['re']))
-        >>> ImmutableObject.get_reconstructor_with_imports(re.compile('abcd', re.I | re.M))
-        ("re.compile('abcd', re.IGNORECASE | re.MULTILINE)", set(['re']))
+        >>> reconstructor, imports = ImmutableObject.get_reconstructor_with_imports(re.compile('abcd'))
+        >>> reconstructor
+        "re.compile('abcd')"
+        >>> imports == set(['re'])
+        True
+        >>> reconstructor, imports = ImmutableObject.get_reconstructor_with_imports(re.compile('abcd', re.I | re.M))
+        >>> reconstructor
+        "re.compile('abcd', re.IGNORECASE | re.MULTILINE)"
+        >>> imports == set(['re'])
+        True
         """
         if isinstance(obj, (int, long, float, str, unicode, types.NoneType)):
             # Bultin types has very convienient representation.
@@ -179,9 +206,6 @@ class UnknownObject(SerializedObject):
     """
     def __init__(self, obj):
         SerializedObject.__init__(self, obj)
-
-        self.module_name = module_name(obj)
-        self.type_name = get_type_name(obj)
         self.partial_reconstructor = UnknownObject.get_partial_reconstructor(obj)
 
     def __repr__(self):
@@ -199,12 +223,17 @@ class UnknownObject(SerializedObject):
 class CompositeObject(SerializedObject):
     """An object of a builtin type that may contain other objects, e.g. a list
     or a dictionary.
-    """
-    def __init__(self, obj):
-        SerializedObject.__init__(self, obj)
 
-        self.module_name = module_name(obj)
-        self.type_name = get_type_name(obj)
+    :IVariables:
+      constructor_format : str
+        A format string that will reconstruct this CompositeObject. Should
+        contain a single %s, which will be replaced with string representation
+        of contained objects, separated with commas.
+        E.g. "[%s]" is a good constructor_format for a list.
+      imports : set
+        Set of imports needed to bring this CompositeObject into current scope.
+    """
+    pass
 
 class SequenceObject(CompositeObject):
     """A builtin object that contains an ordered sequence of other objects
@@ -239,6 +268,9 @@ class SequenceObject(CompositeObject):
             self.constructor_format = self.type_formats_with_imports[type(obj)][0]
             self.imports = self.type_formats_with_imports[type(obj)][1]
 
+    def __repr__(self):
+        return "SequenceObject(%s)" % (self.constructor_format % self.contained_objects)
+
 class MapObject(CompositeObject):
     """A mutable object that contains unordered mapping of key/value pairs.
     """
@@ -249,6 +281,27 @@ class MapObject(CompositeObject):
         self.constructor_format = "{%s}"
         self.imports = set()
 
+class BuiltinException(CompositeObject):
+    """A built-in exception instance, which may hold any kind of objects in its
+    `args` attribute.
+
+    It doesn't matter if an exception was generated by the interpreter or by
+    the user code, it will be serialized as BuiltinException nevertheless. This
+    means we can't assume anything about the contents of `args`.
+
+    User-defined exceptions, both deriving from BaseException and those not,
+    are serialized as UserObjects.
+    """
+    def __init__(self, obj, serialize):
+        CompositeObject.__init__(self, obj)
+
+        self.args = map(serialize, obj.args)
+        self.constructor_format = "%s(%%s)" % class_name(obj)
+        self.imports = set()
+
+        if class_of(obj) in BUILTIN_ENVIRONMENT_ERROR_TYPES and obj.filename is not None:
+            self.args.append(serialize(obj.filename))
+
 def is_immutable(obj):
     # Bool class is a subclass of int, so True and False are included in this
     # condition.
@@ -264,3 +317,13 @@ def is_mapping(obj):
 def is_sequence(obj):
     return isinstance(obj, (array.array, list, frozenset, set,
                             sets.ImmutableSet, sets.Set, tuple))
+
+def is_builtin_exception(obj):
+    """Return True if given object is an instance of a built-in exception, like
+    NameError or EOFError. Return False for instances of user-defined
+    exceptions.
+    """
+    return class_of(obj) in BUILTIN_EXCEPTION_TYPES
+
+def is_serialized_string(obj):
+    return isinstance(obj, ImmutableObject) and obj.type_name == 'str'
