@@ -1,7 +1,7 @@
 import unittest
 
 class TestSomeClass(unittest.TestCase):
-    def test_object_initialization(self):
+    def test___init__(self):
         assert True # implemented test case
 
     def test_some_method(self):
@@ -9,6 +9,8 @@ class TestSomeClass(unittest.TestCase):
 
 class TestNewClass(unittest.TestCase):
     def test_new_method(self):
+        # new_class = NewClass()
+        # self.assertEqual(expected, new_class.new_method())
         assert False # TODO: implement your test here
 
 if __name__ == '__main__':
