@@ -1,19 +1,18 @@
 import os
 import shutil
-import sys
 import tempfile
 import types
 import warnings
 
 from StringIO import StringIO
 
+from pythoscope.astbuilder import EmptyCode
 from pythoscope.generator import add_tests_to_project
 from pythoscope.logger import DEBUG, INFO, get_output, log, set_output
-from pythoscope.store import CodeTreesManager, CodeTreeNotFound, Execution, \
-    Function, ModuleNotFound, PointOfEntry, Project
-from pythoscope.compat import set
-from pythoscope.util import quoted_block, read_file_contents, \
-    write_content_to_file
+from pythoscope.code_trees_manager import CodeTreesManager, CodeTreeNotFound
+from pythoscope.store import Execution, Function, ModuleNotFound, \
+    PointOfEntry, Project
+from pythoscope.util import read_file_contents, write_content_to_file
 
 
 UNPICKABLE_OBJECT = types.ClassType('class', (), {})
@@ -77,12 +76,18 @@ def ProjectInDirectory(project_path):
 
 def with_modules(project, paths, create_files=True):
     for path in paths:
-        project.create_module(os.path.join(project.path, path))
+        project.create_module(os.path.join(project.path, path), code=EmptyCode())
         if create_files:
             putfile(project.path, path, "")
     return project
 Project.with_modules = with_modules
 
+def with_module(project, path, objects=[]):
+    project.create_module(os.path.join(project.path, path),
+      objects=objects, code=EmptyCode())
+    return project
+Project.with_module = with_module
+
 def with_points_of_entry(project, paths):
     poe_path = putdir(project.path, P(".pythoscope/points-of-entry"))
     for path in paths:
@@ -90,10 +95,16 @@ def with_points_of_entry(project, paths):
     return project
 Project.with_points_of_entry = with_points_of_entry
 
+def with_point_of_entry(project, path, content):
+    poe_path = putdir(project.path, P(".pythoscope/points-of-entry"))
+    putfile(poe_path, path, content)
+    return project
+Project.with_point_of_entry = with_point_of_entry
+
 def ProjectWithModules(paths, project_type=EmptyProject):
     project = project_type()
     for path in paths:
-        project.create_module(os.path.join(project.path, path))
+        project.create_module(os.path.join(project.path, path), code=EmptyCode())
     return project
 
 def TestableProject(path, more_modules=[]):
@@ -121,7 +132,7 @@ def generate_single_test_module(template='unittest', **module_kwds):
     """Return test module contents generated for given module.
     """
     project = EmptyProject()
-    project.create_module("module.py", **module_kwds)
+    project.create_module("module.py", code=EmptyCode(), **module_kwds)
     add_tests_to_project(project, ["module.py"], template, False)
     return get_test_module_contents(project)
 generate_single_test_module.__test__ = False
@@ -130,14 +141,6 @@ def get_test_cases(project):
     return list(project.iter_test_cases())
 get_test_cases.__test__ = False
 
-def last_exception_as_string():
-    exc_type, exc_value = sys.exc_info()[:2]
-    # Special case for string exceptions.
-    if isinstance(exc_type, str):
-        return exc_type
-    else:
-        return repr(exc_value)
-
 ###############################################################################
 # Temporary directories/files helpers
 
