@@ -1,7 +1,9 @@
+import os.path
 import sys
 
 from nose import SkipTest
 
+from pythoscope.inspector.static import inspect_code
 from pythoscope.inspector.dynamic import inspect_code_in_context, \
     inspect_point_of_entry
 from pythoscope.serializer import ImmutableObject, MapObject, UnknownObject, \
@@ -9,11 +11,12 @@ from pythoscope.serializer import ImmutableObject, MapObject, UnknownObject, \
 from pythoscope.store import Class, Execution, Function, \
     GeneratorObject, Method, UserObject, Project
 from pythoscope.compat import all
-from pythoscope.util import findfirst, generator_has_ended
+from pythoscope.util import findfirst, generator_has_ended, \
+    last_exception_as_string
 
 from assertions import *
-from helper import TestableProject, PointOfEntryMock, EmptyProjectExecution, \
-    last_exception_as_string, IgnoredWarnings, putfile, TempDirectory
+from helper import ProjectInDirectory, PointOfEntryMock, EmptyProjectExecution, \
+    IgnoredWarnings, putfile, TempDirectory, P, CapturedLogger
 
 
 ########################################################################
@@ -901,8 +904,9 @@ class TestTraceExec:
 
 class TestInspectPointOfEntry(TempDirectory):
     def _init_project(self, module_code="", poe_content=""):
-        self.project = TestableProject(self.tmpdir)
+        self.project = ProjectInDirectory(self.tmpdir)
         putfile(self.project.path, "module.py", module_code)
+        inspect_code(self.project, os.path.join(self.project.path, "module.py"), module_code)
         self.poe = PointOfEntryMock(self.project, content=poe_content)
 
     def test_properly_gathers_all_input_and_output_values_of_a_function_call(self):
@@ -934,3 +938,11 @@ class TestInspectPointOfEntry(TempDirectory):
         inspect_point_of_entry(self.poe)
 
         assert 'module' not in sys.modules
+
+class TestInspectPointOfEntryWithCapturedLog(TempDirectory, CapturedLogger):
+    def test_changes_current_directory_to_the_projects_root(self):
+        project = ProjectInDirectory(self.tmpdir)
+        poe = PointOfEntryMock(project,
+            content="import os; assert os.getcwd() == '%s'" % project.path)
+        assert_not_raises(AssertionError, lambda: inspect_point_of_entry(poe))
+
