@@ -1,24 +1,17 @@
 import os
 
+from pythoscope.astbuilder import parse, EmptyCode
 from pythoscope.generator import TestMethodDescription, TestGenerator
-from pythoscope.generator.adder import add_test_case_to_project, \
-    find_test_module, module_path_to_test_path
+from pythoscope.generator.adder import add_test_case_to_project, add_test_case, \
+    find_test_module, module_path_to_test_path, replace_test_case
 from pythoscope.inspector.static import inspect_code
 from pythoscope.store import TestClass, TestMethod
 
 from assertions import *
+from factories import create
 from helper import get_test_cases, CapturedLogger, CustomSeparator, \
     EmptyProject, ProjectWithModules
 
-# Let nose know that this aren't test classes and functions.
-TestClass.__test__ = False
-TestMethod.__test__ = False
-TestMethodDescription.__test__ = False
-TestGenerator.__test__ = False
-module_path_to_test_path.__test__ = False
-add_test_case_to_project.__test__ = False
-find_test_module.__test__ = False
-
 
 def ProjectAndTestClass(test_module_name):
     project = ProjectWithModules(["module.py", test_module_name])
@@ -34,6 +27,18 @@ class TestModulePathToTestPathWithCustomSeparator(CustomSeparator):
                      module_path_to_test_path("pythoscope#__init__.py"))
 
 class TestGeneratorAdder:
+    def test_replacing_a_test_case_removes_it_from_the_list_of_objects_and_list_of_test_cases(self):
+        project = EmptyProject()
+        module = project.create_module("module.py", code=parse("# only comments"))
+        test_class = create(TestClass, name="TestSomething")
+        new_test_class = create(TestClass, name="TestSomethingElse")
+        add_test_case(module, test_class)
+
+        replace_test_case(module, test_class, new_test_class)
+
+        assert_equal([new_test_class], module.objects)
+        assert_equal([new_test_class], module.test_cases)
+
     def test_finds_associated_test_modules_that_use_different_name_conventions(self):
         test_module_names = ["test_module.py", "testModule.py", "TestModule.py",
                              "tests_module.py", "testsModule.py", "TestsModule.py",
@@ -58,26 +63,26 @@ class TestGeneratorAdder:
         assert project[test_module_name] is find_test_module(project, test_class)
 
     def test_adds_new_test_methods_to_existing_test_classes_inside_application_modules(self):
-        project = EmptyProject()
-        test_class = TestClass("TestSomething")
-        module = project.create_module("somethings.py")
-        module.add_test_case(test_class)
+        project = EmptyProject().with_module("somethings.py")
+        test_class = create(TestClass, name="TestSomething")
+        add_test_case(project["somethings"], test_class)
 
-        new_test_method = TestMethod("test_new_method")
-        new_test_class = TestClass("TestSomething", test_cases=[new_test_method])
-        add_test_case_to_project(project, new_test_class)
+        method = create(TestMethod)
+        generated_test_class = create(TestClass, name="TestSomething",
+          test_cases=[method])
+        add_test_case_to_project(project, generated_test_class)
 
         assert_length(get_test_cases(project), 1)
-        assert_equal_sets([new_test_method], test_class.test_cases)
-        assert new_test_method.parent is test_class
+        assert_equal_sets([method], test_class.test_cases)
+        assert method.parent is test_class
 
 class TestGeneratorAdderForProjectWithTestModule(CapturedLogger):
     def setUp(self):
         CapturedLogger.setUp(self)
         self.project = EmptyProject()
-        self.existing_test_class = TestClass("TestSomething")
-        self.test_module = self.project.create_module("test_module.py")
-        self.test_module.add_test_case(self.existing_test_class)
+        self.existing_test_class = create(TestClass, name="TestSomething")
+        self.test_module = self.project.create_module("test_module.py", code=EmptyCode())
+        add_test_case(self.test_module, self.existing_test_class)
 
     def _associate_module_with_existing_test_class(self):
         self.associated_module = self.project.create_module("module.py")
@@ -87,28 +92,29 @@ class TestGeneratorAdderForProjectWithTestModule(CapturedLogger):
         self.project.create_module("irrelevant_test_module.py")
         self._associate_module_with_existing_test_class()
 
-        new_test_class = TestClass("new", associated_modules=[self.associated_module])
+        new_test_class = create(TestClass, name="new",
+          associated_modules=[self.associated_module])
         add_test_case_to_project(self.project, new_test_class)
 
         assert new_test_class in self.test_module.test_cases
 
     def test_doesnt_overwrite_existing_test_classes_by_default(self):
-        test_class = TestClass("TestSomething")
+        test_class = create(TestClass, name="TestSomething")
         add_test_case_to_project(self.project, test_class)
 
         assert_length(get_test_cases(self.project), 1)
 
     def test_adds_new_test_classes_to_existing_test_module(self):
-        test_class = TestClass("TestSomethingNew",
-                               associated_modules=[self.project.create_module("module.py")])
+        test_class = create(TestClass, name="TestSomethingNew",
+          associated_modules=[self.project.create_module("module.py")])
         add_test_case_to_project(self.project, test_class)
 
         assert_equal_sets([self.existing_test_class, test_class],
                           get_test_cases(self.project))
 
     def test_adds_new_test_methods_to_existing_test_classes(self):
-        test_method = TestMethod("test_new_method")
-        test_class = TestClass("TestSomething", test_cases=[test_method])
+        test_method = create(TestMethod, name="test_new_method")
+        test_class = create(TestClass, name="TestSomething", test_cases=[test_method])
         add_test_case_to_project(self.project, test_class)
 
         assert_length(get_test_cases(self.project), 1)
@@ -119,7 +125,7 @@ class TestGeneratorAdderForProjectWithTestModule(CapturedLogger):
                              "Adding generated test_new_method to TestSomething in test_module.py.")
 
     def test_after_adding_new_test_case_to_class_its_module_is_marked_as_changed(self):
-        self.existing_test_class.add_test_case(TestMethod("test_something_new"))
+        add_test_case(self.existing_test_class, create(TestMethod, name="test_something_new"))
 
         assert self.test_module.changed
 
@@ -130,16 +136,18 @@ class TestGeneratorAdderForProjectWithTestModule(CapturedLogger):
         assert_equal(['new_import'], self.test_module.imports)
 
     def test_doesnt_overwrite_existing_test_methods_by_default(self):
-        test_method = TestMethod("test_method")
-        test_class = TestClass("TestSomething", test_cases=[test_method])
+        test_method = create(TestMethod, name="test_method")
+        test_class = create(TestClass, name="TestSomething",
+          test_cases=[test_method])
         add_test_case_to_project(self.project, test_class)
 
         assert_equal([test_method],
                      get_test_cases(self.project)[0].test_cases)
 
         # Let's try adding the same method again.
-        new_test_method = TestMethod("test_method")
-        new_test_class = TestClass("TestSomething", test_cases=[new_test_method])
+        new_test_method = create(TestMethod, name="test_method")
+        new_test_class = create(TestClass, name="TestSomething",
+          test_cases=[new_test_method])
         add_test_case_to_project(self.project, new_test_class)
 
         assert_equal([test_method],
@@ -149,8 +157,8 @@ class TestGeneratorAdderForProjectWithTestModule(CapturedLogger):
                              "Test case TestSomething.test_method already exists in test_module.py, skipping.")
 
     def test_overwrites_existing_test_methods_with_force_option(self):
-        test_method = TestMethod("test_method")
-        test_class = TestClass("TestSomething", test_cases=[test_method])
+        test_method = create(TestMethod, name="test_method")
+        test_class = create(TestClass, name="TestSomething", test_cases=[test_method])
         add_test_case_to_project(self.project, test_class)
 
         assert_equal([test_method],
@@ -158,8 +166,9 @@ class TestGeneratorAdderForProjectWithTestModule(CapturedLogger):
 
         # Let's try adding the same method again with a force option
         # set to True.
-        new_test_method = TestMethod("test_method")
-        new_test_class = TestClass("TestSomething", test_cases=[new_test_method])
+        new_test_method = create(TestMethod, name="test_method")
+        new_test_class = create(TestClass, name="TestSomething",
+          test_cases=[new_test_method])
         add_test_case_to_project(self.project, new_test_class, force=True)
 
         # The class is still the same.
@@ -172,6 +181,25 @@ class TestGeneratorAdderForProjectWithTestModule(CapturedLogger):
         assert_contains_once(self._get_log_output(),
                              "Replacing TestSomething.test_method from test_module.py with generated version.")
 
+    def test_should_not_touch_modules_with_errors(self):
+        project = EmptyProject()
+        module = project.create_module("module.py")
+        test_module = project.create_module("test_module.py",
+            errors=[Exception()])
+        add_test_case_to_project(project,
+            create(TestClass, associated_modules=[module]))
+        assert test_module.changed is False
+
+    def test_should_emit_warning_when_trying_to_add_test_to_module_with_errors(self):
+        project = EmptyProject()
+        module = project.create_module("module.py")
+        project.create_module("test_module.py", errors=[Exception()])
+        add_test_case_to_project(project,
+            create(TestClass, name="FooTest", associated_modules=[module]))
+        assert_contains_once(self._get_log_output(),
+            "WARNING: Not adding FooTest to test_module.py, because "
+            "of a failed inspection.")
+
 class TestGeneratorAdderOnCode:
     def setUp(self):
         self.generator = TestGenerator()
